package explorviz.hpc_monitoring.filter.reconstruction;

import java.util.Comparator;
import java.util.Iterator;
import java.util.TreeSet;

import explorviz.hpc_monitoring.record.HostApplicationMetaData;
import explorviz.hpc_monitoring.record.event.AbstractOperationEvent;
import explorviz.hpc_monitoring.record.event.normal.AfterFailedOperationEvent;
import explorviz.hpc_monitoring.record.event.normal.AfterOperationEvent;
import explorviz.hpc_monitoring.record.event.normal.BeforeOperationEvent;
import explorviz.hpc_monitoring.record.trace.Trace;

public class TraceBuffer {
	private static final Comparator<AbstractOperationEvent> COMPARATOR = new AbstractOperationEventComperator();

	private HostApplicationMetaData traceMetadata;
	private final TreeSet<AbstractOperationEvent> events = new TreeSet<AbstractOperationEvent>(
			COMPARATOR);

	private boolean closeable;
	private boolean damaged;

	private int openEvents;
	private int maxOrderIndex = -1;
	private long maxLoggingTimestamp = -1;

	public final long getMaxLoggingTimestamp() {
		return maxLoggingTimestamp;
	}

	public final void insertEvent(final AbstractOperationEvent event) {
		setMaxLoggingTimestamp(event);
		final int orderIndex = setMaxOrderIndex(event);

		if (event instanceof BeforeOperationEvent) {
			if (orderIndex == 0) {
				closeable = true;
			}
			openEvents++;
		} else if (event instanceof AfterOperationEvent) {
			openEvents--;
		} else if (event instanceof AfterFailedOperationEvent) {
			openEvents--;
		}

		if (!events.add(event)) {
			System.out.println("Duplicate entry for orderIndex " + orderIndex + " with traceId "
					+ event.getTraceId());
			damaged = true;
		}
	}

	private final void setMaxLoggingTimestamp(final AbstractOperationEvent event) {
		final long loggingTimestamp = event.getLoggingTimestamp();
		if (loggingTimestamp > maxLoggingTimestamp) {
			maxLoggingTimestamp = loggingTimestamp;
		}
	}

	private final int setMaxOrderIndex(final AbstractOperationEvent event) {
		final int orderIndex = event.getOrderIndex();
		if (orderIndex > maxOrderIndex) {
			maxOrderIndex = orderIndex;
		}
		return orderIndex;
	}

	public void setTrace(final HostApplicationMetaData trace) {
		if (traceMetadata != null) {
			damaged = true;
			return;
		}
		traceMetadata = trace;
	}

	public final boolean isFinished() {
		return !isInvalid() && closeable;
	}

	public final boolean isInvalid() {
		return ((openEvents != 0) || ((maxOrderIndex + 1) != events.size()) || events.isEmpty() || damaged);
	}

	public final Trace toTrace() {
		final AbstractOperationEvent[] arrayEvents = new AbstractOperationEvent[events.size()];
		final Iterator<AbstractOperationEvent> iterator = events.iterator();
		int index = 0;
		while (iterator.hasNext()) {
			arrayEvents[index] = iterator.next();
			index++;
		}

		// TODO set runtimes

		return new Trace(traceMetadata, arrayEvents);
	}

	private static final class AbstractOperationEventComperator implements
			Comparator<AbstractOperationEvent> {
		@Override
		public int compare(final AbstractOperationEvent o1, final AbstractOperationEvent o2) {
			return o1.getOrderIndex() - o2.getOrderIndex();
		}
	}
}
