package explorviz.hpc_monitoring.filter.reduction;

import java.util.Comparator;

import explorviz.hpc_monitoring.record.event.AbstractOperationEvent;
import explorviz.hpc_monitoring.record.event.normal.AfterFailedOperationEvent;
import explorviz.hpc_monitoring.record.trace.Trace;

public class TraceComperator implements Comparator<Trace> {
	@Override
	public int compare(final Trace t1, final Trace t2) {
		final AbstractOperationEvent[] recordsT1 = t1.getTraceEvents();
		final AbstractOperationEvent[] recordsT2 = t2.getTraceEvents();

		if ((recordsT1.length - recordsT2.length) != 0) {
			return recordsT1.length - recordsT2.length;
		}

		final int cmpHostnames = t1.getTraceMetadata().getHostname()
				.compareTo(t2.getTraceMetadata().getHostname());
		if (cmpHostnames != 0) {
			return cmpHostnames;
		}

		final int cmpApplicationNames = t1.getTraceMetadata().getApplication()
				.compareTo(t2.getTraceMetadata().getApplication());
		if (cmpApplicationNames != 0) {
			return cmpApplicationNames;
		}

		for (int i = 0; i < recordsT1.length; i++) {
			final AbstractOperationEvent recordT1 = recordsT1[i];
			final AbstractOperationEvent recordT2 = recordsT2[i];

			final int cmpSignature = recordT1.getOperationSignatureId()
					- recordT2.getOperationSignatureId();
			if (cmpSignature != 0) {
				return cmpSignature;
			}

			final int cmpClass = recordT1.getClass().getName()
					.compareTo(recordT2.getClass().getName());
			if (cmpClass != 0) {
				return cmpClass;
			}

			if (recordT1 instanceof AfterFailedOperationEvent) {
				final int cmpError = ((AfterFailedOperationEvent) recordT1).getCause().compareTo(
						((AfterFailedOperationEvent) recordT2).getCause());
				if (cmpError != 0) {
					return cmpClass;
				}
			}
		}

		return 0;
	}
}
