package explorviz.hpc_monitoring.reader;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.ServerSocketChannel;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.lmax.disruptor.EventHandler;
import com.lmax.disruptor.RingBuffer;
import com.lmax.disruptor.dsl.Disruptor;

import explorviz.hpc_monitoring.disruptor.RecordArrayEvent;
import explorviz.hpc_monitoring.disruptor.RecordEvent;
import explorviz.hpc_monitoring.filter.reconstruction.TraceReconstructionFilter;

public final class TCPReader {
	private final int listeningPort;
	private boolean active = true;

	private ServerSocketChannel serversocket;

	private final RingBuffer<RecordArrayEvent> ringBuffer;

	private final List<Thread> threads = new ArrayList<Thread>();

	public TCPReader(final int listeningPort, final EventHandler<RecordEvent> endReceiver) {
		this.listeningPort = listeningPort;

		final ExecutorService exec = Executors.newCachedThreadPool();
		final Disruptor<RecordArrayEvent> disruptor = new Disruptor<RecordArrayEvent>(
				RecordArrayEvent.EVENT_FACTORY, 16, exec);

		@SuppressWarnings("unchecked")
		final EventHandler<RecordArrayEvent>[] eventHandlers = new EventHandler[1];
		eventHandlers[0] = new TraceReconstructionFilter(2 * 1000 * 1000, endReceiver);
		disruptor.handleEventsWith(eventHandlers);
		ringBuffer = disruptor.start();
	}

	public final void read() {
		try {
			open();
			while (active) {
				final Thread thread = new TCPReaderThread(serversocket.accept(), ringBuffer);
				thread.start();
				threads.add(thread);
			}
			serversocket.close();
		} catch (final IOException ex) {
			System.out.println("Error in read() " + ex.getMessage());
		} finally {
			try {
				serversocket.close();
			} catch (final IOException e) {
				System.out.println("Error in read()" + e.getMessage());
			}
		}
	}

	private final void open() throws IOException {
		serversocket = ServerSocketChannel.open();
		serversocket.socket().bind(new InetSocketAddress(listeningPort));
		System.out.println("listening on port " + listeningPort);
	}

	public final void terminate(final boolean error) {
		System.out.println("Shutdown of TCPReader requested.");
		active = false;
	}
}
